
/* Values computed by hgcd. */
struct hgcd
{
  /* Unsigned values. */
  mp_size_t asize; mp_ptr ap;
  mp_size_t bsize; mp_ptr bp;

  /* Allocated space for the below values, for sanity checking. */
  mp_size_t alloc;

  /* Signed values. (Can and should use a common sign bit)? */
  mp_size_t uvsize[4]; mp_ptr uvp[4];  

#if 0
  /* Also of size talloc each */
  mp_size_t talloc;
  mp_ptr t1p;
  mp_ptr t2p;
  
#else
  /* Space for various temporaries. Size at least 2*(asize + 1) */
  mp_size_t tsize; mp_ptr tp;
#endif
};

struct hgcd_2
{
  mp_limb_t a[2];
  mp_limb_t b[2];

  /* If sign = 0, the diagonal elements u,v' are >=0, off-diagonal
     entries v, u' <= 0. If sign < 0, then diagonal <= 0, off-diagonal
     >= 0. */
  int sign;
  /* The absolute values. Always fit in one limb each. */
  mp_limb_t uv[4];
};

int
mpn_hgcd(struct hgcd *hgcd);

int
mpn_lehmer_hgcd(struct hgcd *hgcd);

mp_size_t
mpn_fgcd(mp_ptr rp, mp_ptr ap, mp_size_t asize,
	 mp_ptr bp, mp_size_t bsize);

void
mpn_hgcd_init(struct hgcd *hgcd,
	      mp_size_t asize,
	      mp_ptr space,
	      mp_size_t alloc);

void
mpn_check_hgcd(struct hgcd *hgcd, mp_size_t asize);

mp_size_t
mpn_hgcd_itch(mp_size_t asize);

int
hgcd_wrap(struct hgcd *hgcd, int (*f)(struct hgcd *));

#if WANT_CHECKS
#define mpn_hgcd(x) hgcd_wrap((x), mpn_hgcd)
#define mpn_lehmer_hgcd(x) hgcd_wrap((x), mpn_lehmer_hgcd)
#endif
